/*
    SPDX-FileCopyrightText: 2014 Frederik Gladhorn <gladhorn@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef OKULAR_DEBUG_PDF_H
#define OKULAR_DEBUG_PDF_H

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(OkularPdfDebug)

#endif
